/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.ScriptingBean;


/**
 * This is the interface of the scripting support for the MainView.
 * 
 * @author Nasir Rather
 */
public interface MainViewScriptingBean {

	/**
	 * Selects the nodes identified by <code>name</code>.
	 * 
	 * @param name - String name of the node.
	 * @param selectAllOccurances - whether to select all occurances of nodes with this name or just the first one
	 * @param clearPreviouslySelected - boolean whether to clear previous selection.
	 * @param exactMatch Whether or not match the name exactly
	 * @param caseSensitive Whether or not to consider case in the match.
	 */
	public void selectNodesByName(String name, boolean selectAllOccurances, boolean clearPreviouslySelected, boolean exactMatch, boolean caseSensitive);

	/**
	 * Selects the root nodes.
	 */
	public void selectRoots();

	/**
	 * Selects all nodes of a specific type
	 * @param type see the node filter dialog to see the available node types  
	 * @param clearPreviouslySelected
	 */
	public void selectNodesByType(String type, boolean clearPreviouslySelected);

	/**
	 * De-selects the currently selected nodes, and selects their unselected siblings instead.
	 */
	public void selectInverseSiblings();

	/**
	 * Shows the children (if any) of the selected node(s).
	 */
	public void openSelectedNodes();

	/**
	 * Closes the selected node(s) (hides the children if showing). 
	 */
	public void closeSelectedNodes();

	/**
	 * Open all descendants of the selected node(s).
	 */
	public void openAllDescendents();

	/**
	 * Close all descendants of the selected node(s).
	 */
	public void closeAllDescendents();

	/**
	 * Change the child-parent relationships.
	 * @param cprels new child-parent relationships which defines the nested hierarchy
	 */
	public void setHierarchy(String[] cprels);

	/**
	 * Change the display of labels.
	 * <p> {@link ScriptingConstants#SCALE_BY_NODE_SIZE} produces the fastest drawing time
	 * @param mode - one of {@link ScriptingConstants#FIXED}, 
	 * {@link ScriptingConstants#SCALE_BY_NODE_SIZE}, or {@link ScriptingConstants#SCALE_BY_LEVEL}
	 */
	public void setLabelMode(String mode);

	/**
	 * Take a snapShot of the MainView and add it to the Filmstrip.
	 * 
	 * @param askForComments - whether to ask for comments.
	 */
	public void takeSnapShot(boolean askForComments);

	/**
	 * Select all the children of the selected node(s).
	 */
	public void selectAllChildren();

	/**
	 * Select all the descendants of the selected node(s).
	 */
	public void selectAllDescendents();

	/**
	 * Layout children of the selected node(s).
	 * @param newLayoutMode - String new layout mode @see ca.uvic.csr.shrimp.ScriptingConstants
	 * @see ScriptingConstants#GRID_BY_TYPE
	 * @see ScriptingConstants#SPRING
	 * @see ScriptingConstants#TREE_HORIZONTAL
	 * @see ScriptingConstants#TREE_VERTICAL
 	 * @see ScriptingConstants#TREEMAP
	 * @see ScriptingConstants#RADIAL
	 */
	public void layoutChildren(String newLayoutMode);

	/**
	 * Layout selected node(s).
	 * 
	 * @param newLayoutMode - String new layout mode @see ca.uvic.csr.shrimp.ScriptingConstants
	 * @see ScriptingConstants#GRID_BY_TYPE
	 * @see ScriptingConstants#SPRING
	 * @see ScriptingConstants#TREE_HORIZONTAL
	 * @see ScriptingConstants#TREE_VERTICAL
 	 * @see ScriptingConstants#TREEMAP
	 * @see ScriptingConstants#RADIAL
	 */
	public void layoutSelected(String newLayoutMode);

	/**
	 * Filter the selected node(s).
	 */
	public void filterSelectedNodes();

	/**
	 * Group the selected node(s).
	 * @tag Shrimp(grouping)
	 */
	public void groupSelectedNodes();

	/**
	 * Ungroup the selected node(s).
	 * @tag Shrimp(grouping)
	 */
	public void ungroupSelectedNodes();

	/** 
	 * Filters or unfilters all arcs from the display
	 * @param filter show or hide all arcs
	 */
	public void filterAllArcs(boolean filter);

	/** 
	 * Filters or unfilters all nodes from the display
	 * @param filter show or hide all nodes
	 */
	public void filterAllNodes(boolean filter);

	/**
	 * Filters or unfilters arcs of String type from the display
	 * @param type String type of the arcs - see the arc filter dialog for the available arc types
	 * @param filter - boolean whether to show or hide the arcs.
	 */
	public void filterArcsByType(String type, boolean filter);

	/**
	 * Filters or unfilters nodes of String type from the display.
	 * 
	 * @param type type of the nodes - see the node filter dialog for the available arc types
	 * @param filter whether to show or hide the nodes.
	 */
	public void filterNodesByType(String type, boolean filter);

	/**
	 * Filters or unfilters artifacts of a specific type from the data.
	 * Nodes of the same type will be filtered or unfiltered from the display.
	 * @param type
	 * @param filter show or hide artifacts of the given type
	 */
	public void filterArtifactDataType(String type, boolean filter);

	/**
	 * Filters or unfilters relationships of a specific type from the data.
	 * Arcs of the same type will be filtered or unfiltered from the display.
	 * @param type
	 * @param filter show or hide relationships of the given type
	 */
	public void filterRelationshipDataType(String type, boolean filter);

	/**
	 * Filters or unfilters all artifact types from the data.
	 * All nodes will be filtered or unfiltered from the display.
	 * @param filter show or hide all artifacts
	 */
	public void filterAllArtifactDataTypes(boolean filter);

	/**
	 * Filters or unfilters all relationship types from the data.
	 * All arcs will be filtered or unfiltered from the display.
	 * @param filter show or hide all relationships
	 */
	public void filterAllRelationshipDataTypes(boolean filter);

	/**
	 * Unfilter all the previously filtered artifacts.
	 */
	public void unFilterAllArtifacts();

	/**
	 * Zooms back to the root node.
	 */
	public void focusOnHome();

	/**
	 * Redo the action that was previously undone.
	 */
	public void redoAction();

	/**
	 * Undo an action. NOTE: not all actions a user performs can be undone.
	 */
	public void undoAction();

	/**
	 * Sets the zoom mode for the MainView. This mode determines what happens when a user tries to zoom
	 * using the proper input (X, Z keys on the keyboard or the middle mouse button are the defaults).
	 * 
	 * @param newMode the new zoom mode
	 * @see ScriptingConstants#ZOOM
	 * @see ScriptingConstants#MAGNIFY
	 * @see ScriptingConstants#FISHEYE
	 */
	public void changeZoomMode(String newMode);

	/**
	 * Causes the view to navigate to the node with the given name.
	 * If there are multiple nodes in the display with the same name, 
	 * the first one found will be used.
	 * @param name the name of the node to navigate to
	 */
	public void navigateToNode(String name);

	/**
	 * Displays a non-modal dialog with the given message.
	 * @param message the message to display
	 */
	public void showDialog(String message);

	/**
	 * Displays a non-modal dialog with a message and title
	 * @param message the message to display
	 * @param title	title for the dialog
	 */
	public void showDialog(String message, String title);

	/**
	 * Displays a modal dialog with no title
	 * @param message the message to display
	 */
	public void alert(String message);

	/**
	 * Displays a modal dialog with a message.
	 * @param message - String new message to display
	 * @param title	  - String the title of the dialog
	 */
	public void alert(String message, String title);

	/**
	 * Displays a modal dialog with an input field.
	 * @param message the message to display
	 * @param title	the title of the dialog
	 * @return the value the user typed
	 */
	public String prompt(String message, String title);

	/**
	 * Pauses any further script operation for <code>time</code> ms.
	 * 
	 * @param time the time in ms.
	 */
	public void pause(long time);

	/**
	 * Causes the display to refresh.
	 */
	public void refresh();

	/**
	 * Selects all nodes that are connected to the currently selected nodes.
	 */
	public void selectConnectedNodes();

	/**
	 * Creates composite arcs for the given types.
	 * @param arcTypes
	 */
	public void createComposites(String[] arcTypes);

	/**
	 * Removes the composite arcs.
	 */
	public void clearComposites();

}
